import logging, importlib, sys, os

log = logging.getLogger('genthemall.utils')

def load_class(clazz):
    """
    util method for dynamic load class.
    """
    dotIdx = clazz.rindex('.')
    mod = importlib.import_module(clazz[0:dotIdx])
    return getattr(mod, clazz[dotIdx+1:])

def load_function(fn):
    """
    util method for dynamic load function.
    """
    return load_class(fn)
        
def load_command(cmdName):
    """
    util method for dynamic load genthemall command.
    """
    return load_class('genthemall.commands.Command%s%s' % \
                          (cmdName[0].upper(), cmdName[1:]))

def transform_config(config, _type):
    """
    Transform config to some type config.
    """
    try:
        confFn = load_function('genthemall.conf.%s' % _type)
        confFn(config)
    except AttributeError:
        log.error('config type [%s] not found.' % (_type))
        sys.exit(1)


def copyfile(source, dest, buffer_size=1024*1024):
    """
    Copy a file from source to dest. source and dest
    can either be strings or any object with a read or
    write method, like StringIO for example.
    """
    if not hasattr(source, 'read'):
        source = open(source, 'rb')
    if not hasattr(dest, 'write'):
        dest = open(dest, 'wb')

    while 1:
        copy_buffer = source.read(buffer_size)
        if copy_buffer:
            dest.write(copy_buffer)
        else:
            break

    source.close()
    dest.close()

def copyfiles(src, dest, ignore=None):
    if os.path.isdir(src):
        if not os.path.isdir(dest):
            os.makedirs(dest)
        files = os.listdir(src)
        if ignore is not None:
            ignored = ignore(src, files)
        else:
            ignored = set()
        for f in files:
            if f not in ignored:
                copyfiles(os.path.join(src, f), 
                                    os.path.join(dest, f), 
                                    ignore)
    else:
        copyfile(src, dest)
            



import sys, os, logging, json
from optparse import OptionParser
from genthemall.utils import load_command, load_function, transform_config
from genthemall.core import GTLGenerator, GTLTemplateHolder, GTLConfig

log = logging.getLogger('genthemall.command')

class BaseCommand:
    def __init__(self, args):
        self.parser = OptionParser(usage=self._usage, add_help_option=False)
        self.parser.add_option(
            '-f', '--config-file', dest='configFile', 
            default='genthemall.cfg', metavar='PATH',
            help='Sepecify config file for use, e.g. "../other.cfg". Default is genthemall.cfg')
        self.parser.add_option(
            '-v', '--verbose', action='count', dest='verbose',
            help='Increase verbosity (specify multiple times for more).')
        self._args = args

    def add_option_template(self):
        self.parser.add_option(
            '-t', '--template-folder',
            default='./.genthemall', metavar='PATH',
            help='sepecify template folder for use.')

    def add_option_output_folder(self):
        self.parser.add_option(
            '-o', '--output-folder',
            default='./out', metavar='PATH',
            help='Sepecify output folder for the generate files.')

    def init_options(self):
        """
        Parser user options and init logging level
        """
        if not hasattr(self, 'opts'):
            (self.opts, self.args) = self.parser.parse_args(self._args)
            log_level = logging.WARNING # default
            if self.opts.verbose == 1:
                log_level = logging.INFO
            elif self.opts.verbose >= 2:
                log_level = logging.DEBUG
            logging.basicConfig(level=log_level)

    def help(self):
        """
        Print help message.
        """
        self.parser.print_help()

    def execute(self):
        """
        The method for execute, Sub class need implement the method.
        """
        pass

    def do_some_check(self, args_gt_length=0):
        """
        Just do some check, It's ugly, so need change, but now I don't know how, just no idea.
        """
        if not os.path.exists(self.opts.configFile):
            print('Config file [%s] not found. Please check or use genthem project command to create one.' % self.opts.configFile)
            sys.exit(1)
        if len(self.args) < args_gt_length:
            self.help()
            sys.exit(1)

class CommandHelp(BaseCommand):
    _usage = 'genthemall help <command>'

    def __init__(self, args):
        BaseCommand.__init__(self, args)
        self.args = args

    def execute(self):
        if len(self.args) < 2:
            print self._usage
        else:
            cmdClass = load_command(self.args[1])
            cmd = cmdClass(None)
            cmd.help()


class CommandProject(BaseCommand):

    _usage = "%prog project [options] <projectName> <namespace> [displayName] [options]"

    def __init__(self, args):
        BaseCommand.__init__(self, args)

        self.init_options()

    def execute(self):
        if len(self.args) < 3:
            self.help()
            sys.exit(1)
        projectName = self.args[1]
        namespace = self.args[2]
        displayName = ''
        if len(self.args) > 3:
            displayName = self.args[3]

        gtlConfig = GTLConfig(self.opts.configFile)
        conf = gtlConfig.conf
        conf['projectName'] = projectName
        conf['namespace'] = namespace
        conf['displayName'] = displayName
        gtlConfig.save()

class CommandModule(BaseCommand):
    _usage = '%prog module <moduleName> <propertyName> <propertyValue> [options]'

    def __init__(self, args):
        BaseCommand.__init__(self, args)
        self.init_options()

    def execute(self):
        self.do_some_check(args_gt_length=4)
        moduleName = self.args[1]
        propertyName = self.args[2]
        propertyValue = self.args[3]
        gtlConfig = GTLConfig(self.opts.configFile)
        module = gtlConfig.get_module(moduleName)
        module[propertyName] = propertyValue
        gtlConfig.save()
        
class CommandField(BaseCommand):
    
    _usage = '%prog field <moduleName> <filedName> <propertyName>=<propertyValue>... [options]'

    def __init__(self, args):
        BaseCommand.__init__(self, args)
        self.init_options()

    def execute(self):
        self.do_some_check(args_gt_length=4)

        moduleName = self.args[1]
        fieldName = self.args[2]

        gtlConfig = GTLConfig(self.opts.configFile)
        modifyField = gtlConfig.get_field(moduleName, fieldName)
        for p in self.args[3:]:
            prop = p.split('=')
            if len(prop) == 2:
                modifyField[prop[0]] = prop[1]
            else:
                self.help()
                sys.exit(1)
        
        gtlConfig.save()

class CommandRemove(BaseCommand):
    _usage = '%prog remove <moduleName> [fieldName] [options]'

    def __init__(self, args):
        BaseCommand.__init__(self, args)
        self.init_options()

    def execute(self):
        self.do_some_check(args_gt_length=2)
        moduleName = self.args[1]
        fieldName = None
        if len(self.args) > 2:
            fieldName = self.args[2]

        gtlConfig = GTLConfig(self.opts.configFile)
        config = gtlConfig.conf
        module = gtlConfig.get_module(moduleName)

        if fieldName is not None:
            f = gtlConfig.get_field(moduleName, fieldName)
            module.get('fields', []).remove(f)
        else:
            config.get('modules', []).remove(module)
            
        gtlConfig.save()
        
class CommandGenerate(BaseCommand):

    _usage = '%prog generate <templateName> <dest> [options]'
    
    def __init__(self, args):
        BaseCommand.__init__(self, args)
        self.add_option_template()
        self.add_option_output_folder()
        self.parser.add_option(
            '-l', '--one-file', default=False,
            action='store_true',
            help='Just generate a file.')
        self.init_options()

    def execute(self):
        self.do_some_check(args_gt_length=3)
        template = self.args[1]
        dest = self.args[2]
        gtlConfig = GTLConfig(self.opts.configFile)
        config = gtlConfig.conf
        typeIdx = template.find('.')
        if typeIdx == -1:
            log.error('template name must be "type.templatename".')
            sys.exit(1)
        transform_config(config, template[:typeIdx])
        generator = GTLGenerator(
            config=config, template_folder=self.opts.template_folder,
            out_dir=self.opts.output_folder, one_file=self.opts.one_file)
        log.debug('Generator init done, and using config type [%s].' \
                  % template[:typeIdx])
        generator.generate(template, dest)


class CommandTemplate(BaseCommand):

    _usage = """%prog template <command> [args] [options]
  list    List all templates
  edit    edit template using $EDITOR
"""
    
    def __init__(self, args):
        BaseCommand.__init__(self, args)
        self.add_option_template()
        self.init_options()

    def execute(self):
        self.do_some_check(args_gt_length=2)
        cmd = self.args[1]

        holder = GTLTemplateHolder(folder=self.opts.template_folder)
        if cmd == 'list':
            holder.list_templates()
        elif cmd == 'edit':
            if len(self.args) < 3:
                log.error('Please sepecify template name for edit.')
                sys.exit(1)
            tmpl = holder.find_template_by_name(self.args[2])
            if tmpl is None:
                log.warn('Template name [%s] not exists.' % self.args[2])
                sys.exit(1)
            if os.environ.get('EDITOR', None) is not None:
                os.system('$EDITOR %s' % tmpl)
            else:
                log.error('Env EDITOR not set.')
                sys.exit(1)

class CommandPrintConfig(BaseCommand):
    _usage = """%prog printConfig <type> [options]"""
        
    def __init__(self, args):
        BaseCommand.__init__(self, args)
        self.init_options()

    def execute(self):
        self.do_some_check(args_gt_length=2)
        _type = self.args[1]
        gtlConfig = GTLConfig(self.opts.configFile)
        config = gtlConfig.conf
        transform_config(config, _type)
        print(json.dumps(config, indent=4))



#!/usr/bin/env python

import sys, logging, os, importlib
from optparse import OptionParser
from genthemall.utils import load_command

USAGE = """usage: genthemall <command> [<args>]

 project     Add or modify project property.
 module      Add or modify module command property.
 field       Add or modify field properties.
 remove      Remove specify module or field.
 generate    generate files using MakoTemplate.
 template    List or edit some template file.
 printConfig Print sepecify type config.
 help        show command help info.
"""

def main():
    """
    We begin here.
    """
    if len(sys.argv) < 2:
        print(USAGE)
        sys.exit(1)

#    logging.basicConfig(level=logging.INFO)
    cmdName = sys.argv[1]
    try:
        CmdClass = load_command(cmdName)
        cmd = CmdClass(sys.argv[1:])
        cmd.execute()
    except AttributeError:
        print(USAGE)
        sys.exit(1)





from genthemall.core import GTLTemplateHolder, GTLGenerator
from nose.tools import eq_, ok_

def test_gtl_generator():
    g = GTLGenerator('tests/genthemall.cfg')
    g.print_config()
    ok_(g.generate('create_database,java_base_model'))
    ok_(g.config is not None)

def test_find_templates():
    tmpl = GTLTemplateHolder('genthemall')
    ok_(len(tmpl.sys_templates) > 0)
    ok_(len(tmpl.user_templates) == 0)



def _to_camel_name(name):
    """
    Convert name to camel name.
    example:
    SysUser will convert to sysUser
    SysRole will convert to sysRole
    """
    if name is not None and len(name) > 1:
        return name[0].lower() + name[1:]
    return name

def _to_pascal_name(name):
    """
    Convert name to pascal name.
    example:
    sysUser will convert to SysUser
    sysRole will convert to SysRole
    """
    if name is not None and len(name) > 1:
        return name[0].upper() + name[1:]
    return name

def _to_upper_name(name):
    """
    Convert name to uppercase name. 
    example: 
    sysUser will convert to SYS_USER
    sysRole will convert to SYS_ROLE
    """
    if name is not None and len(name) > 1:
        u_name = ''
        for s in name:
            if s == s.upper():
                u_name += '_' + s
            else:
                u_name += s.upper()
        return u_name
    return name


default_max_map = {
    'long': 1 << 64,
    'int': 1 << 32,
    'string': 256,
    'text': 1024,
    'date': None,
}

default_min_map = {
    'long': -1 << 64,
    'int': -1 << 32,
    'string': 0,
    'text': 0,
    'date': None,
}

java_type_map = {
        'long': 'java.lang.Long',
        'int': 'java.lang.Integer',
        'string': 'java.lang.String',
        'text': 'java.lang.String',
        'date': 'java.util.Date',
}
java_short_type_map = {
        'long': 'long',
        'int': 'int',
        'string': 'String',
        'text': 'String',
        'date': 'java.util.Date',
}
database_type_map = {
        'long': 'NUMBER',
        'int': 'NUMBER',
        'string': 'VARCHAR2',
        'text': 'VARCHAR2',
        'date': 'DATETIME',
}

def java(config):
    """
    Put the config into this method, and make it to java config.
    """
    config['package'] = '.'.join(config['namespace'].split('.')[::-1])
    config['path'] = config['package'].replace('.', '/')
    for m in config.get('modules', []):
        m.setdefault('camelName', _to_camel_name(m['name']))
        m.setdefault('pascalName', _to_pascal_name(m['name']))
        m.setdefault('displayName', _to_pascal_name(m['name']))
        for f in m.get('fields', []):
            f.setdefault('camelName', _to_camel_name(f['name']))
            f.setdefault('required', True)
            f.setdefault('pascalName', _to_pascal_name(f['name']))
            f.setdefault('getter', 'get%s' % f.get('pascalName'))
            f.setdefault('setter', 'set%s' % f.get('pascalName'))
            f.setdefault('formatSymbol', '%d' if f.get('type') in \
                         ['int', 'long'] else '%s')
            f.setdefault('javaType', java_type_map[f.get('type')])
            f.setdefault('javaShortType', java_short_type_map[f.get('type')])
            f.setdefault('min', default_min_map[f.get('type')])
            f.setdefault('max', default_max_map[f.get('type')])
            f.setdefault('isId', True if f['name'] == 'id' else False)
            f.setdefault('order', -1 if f['isId'] else 0)
            f.setdefault('editable', False if f['isId'] else True)
            f.setdefault('listable', False if f['isId'] else True)

        m.get('fields').sort(lambda a, b: a['order'] - b['order'])
    return config

def oracle(config):
    """
    Put the config into this method, and make it to oracle config.
    """
    for m in config.get('modules', []):
        m.setdefault('tableName', _to_upper_name(m['name']))
        m.setdefault('pascalName', _to_pascal_name(m['name']))
        m.setdefault('camelName', _to_camel_name(m['name']))
        for f in m.get('fields', []):
            f.setdefault('required', True)
            f.setdefault('columnName', _to_upper_name(f['name']))
            f.setdefault('databaseType', \
                         database_type_map[f.get('type', 'string')])
            f.setdefault('max', default_max_map[f.get('type', 'string')])
            f.setdefault('isId', True if f['name'] == 'id' else False)
            f.setdefault('order', -1 if f['isId'] else 0)

        m.get('fields').sort(lambda a, b: a['order'] - b['order'])        
    return config




#!/usr/bin/env python

import sys, os

from setuptools import setup, find_packages

from genthemall.core import get_version

# Find template files and package them
data_files = []
for dirpath, dirnames, filenames in os.walk('genthemall'):
    files = []
    for f in filenames:
        if f.endswith('.gt'):
            files.append(os.path.join(dirpath, f))
    if files:
        data_files.append((dirpath, files))


setup (
    name = 'GenThemAll',
    version = get_version(),
    description = 'GenThemAll is a simple, Pythonic tool for genernate code and whatevery you want.',
    author = 'Doni',
    author_email = 'd@ii2d.com',
    url = 'https://github.com/donilan/python-genthemall',
    license = 'Apache License, Version 2.0',
    packages = find_packages(),
    test_suite = 'nose.collector',
    tests_require = ['nose'],
    install_requires = ['config', 'mako'],
    entry_points = {
        'console_scripts': [
            'genthemall = genthemall.main:main',
        ]
    },
    data_files = data_files,
    classifiers = [
        'Environment :: Console',
        'Programming Language :: Python :: 2.7',
        'License :: OSI Approved :: Apache Software License'
    ],
)



import logging, os, re, io, sys, json
from mako.template import Template as MakoTemplate
from config import Config
from genthemall.utils import copyfiles

__version__ = '0.3.0'

def get_version():
    return __version__

log = logging.getLogger('genthemall.core')

class GTLTemplateHolder():
    
    def __init__(self, folder = './.genthemall'):
        self.folder = folder
        self.templates = []
        self._find_template_files()

    def copy_templates_to_workspace(self):
        src = os.path.join(os.path.abspath(os.path.dirname(__file__)), 'gt')
        log.debug('Copy [%s] to [%s]' % (src, self.folder))
        copyfiles(src, self.folder)

    def _find_template_files(self):
        """Find template files ."""
        if not os.path.exists(self.folder):
            log.info('Template folder [%s] not found, then create and copy system template in it.' % self.folder)
            self.copy_templates_to_workspace()
        self.templates = []
        for (root, dirs, files) in os.walk(self.folder):
            for f in files:
                if f.endswith('.gt'):
                    self.templates.append((f[:-3], os.path.join(root, f)))

    def list_templates(self):
        """List all template we found, and print them on terminal."""
        print 'GenThemAll %s List all templates.\n' % get_version()
        for idx, t in enumerate(self.templates):
            print '%02d. %30s' % ((idx+1), t[0])

    def find_template_by_name(self, name):
        """Find sepecify template by name."""
        for t in self.templates:
            if t[0] == name:
                return t[1]
        return None

    def find_all_templates(self):
        return [t[1] for t in self.templates]

### GTLTemplateHolder

class GTLGenerator():
    """GTLGenerator"""
    def __init__(self, config, template_folder = None, gtl_template_holder = None\
                 , out_dir = 'out', one_file = False):
        self.outDir = out_dir
        self.config = config
        self.oneFile = one_file
        if gtl_template_holder and isinstance(gtl_template_holder \
            , GTLTemplateHolder):
            self.gtlTemplateHolder = gtl_template_holder
        else:
            self.gtlTemplateHolder = GTLTemplateHolder(template_folder)

    def generate(self, tmpl_name, dest):
        """Generate with sepecify template names."""
        if self.config is None or self.gtlTemplateHolder is None:
            log.error('Config load fail or Template load fail.')
            sys.exit(1)

        template = self.gtlTemplateHolder.find_template_by_name(tmpl_name)
        if template is None:
            log.error('Template name[%s] not found' % tmpl_name)
            sys.exit(1)

        dest = os.path.join(self.outDir, dest)

        # Just generate a file if oneFile be setted.
        if self.oneFile:
            self._generate_to_file(template, dest, config=self.config)
            return

        for module in self.config.get('modules', []):
            self._generate_to_file(template, dest, config=self.config, module=module)


    def _generate_to_file(self, tmpl_file_name, dest, **data):
        out = MakoTemplate(dest).render(**data)
        destDir = os.path.dirname(out)
        if not os.path.exists(destDir):
            log.info('Dir [%s] not exists, create.' % destDir)
            os.makedirs(destDir)
        log.info('Generating [%s]' % out)
        tmpl = MakoTemplate(filename=tmpl_file_name)
        content = tmpl.render(**data)
        file = open(out, 'w')
        file.write(content)
        file.close()
### class GTLGenerator


### class GTLConfig
class GTLConfig:
    
    def __init__(self, config_file):
        self.configFile = config_file
        content = '{}'
        if os.path.exists(config_file):
            f = open(config_file)
            content = f.read()
            if len(content) == 0:
                content = '{}'
            f.close()
        self.conf = json.loads(content)
            
    def save(self):
        """
        Save config to file.
        """
        f = open(self.configFile, 'w')
        f.write(json.dumps(self.conf, indent=4))
        f.close()

    def get_module(self, moduleName):
        """
        Get module by module name from config file.
        """
        modules = self.conf.setdefault('modules', [])
        modifyModule = None
        for module in modules:
            if module.get('name') == moduleName:
                modifyModule = module
                break
        if modifyModule is None:
            modifyModule = {}
            modules.append(modifyModule)
            modifyModule['name'] = moduleName
        return modifyModule

    def get_field(self, moduleName, fieldName):
        """
        Get field by module name and field name from config file.
        """
        modifyModule = self.get_module(moduleName)
        fields = modifyModule.setdefault('fields', [])
        modifyField = None
        for f in fields:
            if f.get('name', '') == fieldName:
                modifyField = f
                break
        if modifyField is None:
            modifyField = {}
            modifyField['name'] = fieldName
            fields.append(modifyField)
        return modifyField
### END class GTLConfig
